import javax.swing.*;
import java.awt.*;
import java.awt.Color;

// Implementation specific graphics belong here.
class Visual extends JFrame { // We might as well inherit JFrame directly..

	public Visual(String windowname, Block[] blocks, int width, int height) {
		super(windowname);
		d_blocks = blocks; // We're assigning the same array here for easy access
		d_width = width; d_height = height;
		
		d_cursor = (d_height/2)*d_width + d_width/2; // In the middle

		// We need to add the overloaded JPanel to our content
		d_content.add(d_panel);

		// If the visual is closed, the program exits.
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		setResizable(false);
		pack();
		show();
	}

	// This refreshes the board
	public void refresh() { d_panel.repaint(); } // This isn't fast enough

	// Cursor changes go here
	public void setCursor(int i) {
		d_cursor = i;
		refresh();
	}

	// Shows the score and quits
	public void reportScore(int score) {
		JOptionPane.showMessageDialog(this, "Game over.  Your score: " + score);
		System.exit(0);
	}

	
	/* Data */
	private Container d_content = getContentPane();
		
	// Instead of creating a whole new class for this, we're just introducing it as JPanel and overloading a couple of methods
	private JPanel d_panel = new JPanel() {

		// Used for actual drawing
		public void paintComponent(Graphics g) {
			// Although this does nothing important for our purposes, the idea is to keep the original functionality and add some own.
			super.paintComponent(g);
			
			// Borders
			g.setColor(Color.BLACK);
			g.fillRect(0, 0, d_borders, d_height*d_blocksize+d_borders);
			g.fillRect(d_width*d_blocksize + d_borders, 0, d_borders, d_height*d_blocksize+d_borders);
			g.fillRect(d_borders, d_height*d_blocksize, d_height*d_blocksize, d_borders);

			// Going through the blocks and drawing
			for (int x = 0; x < d_width; ++x)
				for (int y = 0; y < d_height; ++y) {
					int pos_x = x*d_blocksize + d_borders; 
					int pos_y = (d_height - y - 1)*d_blocksize; // Y is reversed.  Block(0,0) resides at the bottom of the board.
				   
					switch (d_blocks[y*d_width + x].getColor()) { // We could avoid repeating the code, but it's not worth it
						case BLUE:
							d_ii_blue.paintIcon(this, g, pos_x, pos_y);
							break;
						case GREEN:
							d_ii_green.paintIcon(this, g, pos_x, pos_y);
							break;
						case PINK:
							d_ii_pink.paintIcon(this, g, pos_x, pos_y);
							break;
						case RED:
							d_ii_red.paintIcon(this, g, pos_x, pos_y);
							break;
						case YELLOW:
							d_ii_yellow.paintIcon(this, g, pos_x, pos_y);
							break;
					}
				}

			// Drawing the cursor
			int cursorsize = 36;
			int pos_y = (d_height-(d_cursor/d_width)-1)*d_blocksize - (cursorsize-d_blocksize)/2; 
			int pos_x = 5 + (d_cursor%d_width)*d_blocksize - (cursorsize-d_blocksize)/2; // Border + position - half of the cursor border
		   	d_ii_cursor.paintIcon(this, g, pos_x, pos_y); 	
		}

		public Dimension getPreferredSize() {
			// The board is d_width * d_height blocks, borders are d_borders pixels.  No border at the top
			return new Dimension(d_width*d_blocksize + 2*d_borders,
					d_height*d_blocksize + d_borders);
		}
	};

	private final int d_borders = 5;
	private final int d_blocksize = 32;
	private Block[] d_blocks; // This is the exactly same array as in Game
	private int d_width, d_height; // In blocks
	private int d_cursor;

	// The actual images	
	private final ImageIcon d_ii_blue = new ImageIcon("res/blue.png"),
			d_ii_green = new ImageIcon("res/green.png"),
			d_ii_pink = new ImageIcon("res/pink.png"),
			d_ii_red = new ImageIcon("res/red.png"),
			d_ii_yellow = new ImageIcon("res/yellow.png"),
			d_ii_cursor = new ImageIcon("res/cursor.png");
}
